//#define DEBUG_LOG
//#define _PIVOT_OFF_


#define DIMENSIONS 2
#include <numeric>
#include <algorithm>
#include <ctime>
#include <limits>

// default is no gravity
#define GRAVITY_AXES -100 // -100=off, 0=x, 1=y
#define GRAVITY 9.81
#define RHO 1.0

#include "gw_assembler.hh"

using namespace std;

int main()
{
  // setup grid
  const int dim = DIMENSIONS;
  Vector extend( dim, 100.0 );				// 100.0m x 100.0m
  std::vector<size_t> nCells( dim, 50 );		// 50 x 50 Elements
  Grid grid( extend, nCells );

  string tif_filename = "test05.tif";
  //string tif_filename = "test50.tif";
  string cfg_filename = "gw_parameters.cfg";

  // setup parameters
  BoundaryConditions bc( grid, cfg_filename.c_str() );

  typedef SpatialParameters ConductivityCoefficients;
  ConductivityCoefficients conductivity( tif_filename.c_str(), grid, cfg_filename.c_str(), "Conductivity" );

  Sources sources;

  // setup assembler
  GroundwaterAssembler assembler( grid, conductivity, sources, bc );

  // setup matrix and rhs
  typedef GroundwaterAssembler::Matrix Matrix;
  const size_t grid_size = std::accumulate( nCells.begin(), nCells.end(), 1, std::multiplies<size_t>() );
  std::cout << "grid_size = " << grid_size << std::endl;

  Matrix A( grid_size, grid_size, 0.0 );
  Vector b( grid_size, 0.0 );

  // setup clock
  clock_t timer_begin = clock();
  clock_t timer;
  assembler.assemble( A, b );

  timer = clock();
  cout << "Time for assembling: " << (double)(timer-timer_begin)/CLOCKS_PER_SEC << endl;
  timer_begin = clock();

  //std::cout << std::endl;
  //std::cout << "b = " << b << std::endl;
  //b.print();

  if(grid.cells()[0]<10){
    cout << "Matrix: " << A << endl;
  }

  Vector x( grid_size, 0.0 );

  cout << "solving linear system..." << endl;
  A.gauss_solver( x, b );
  
  timer = clock();
  cout << "Time for solving: " << (double)(timer-timer_begin)/CLOCKS_PER_SEC << endl;
  timer_begin = clock();

  if( grid.cells()[0]<10 )
    {
      cout << "Solution: " << endl;
      cout << x << endl;
    }

  // output to image in hsv color space
  vector<unsigned int> cells( 3, 1 );
  for( unsigned int i=0; i<Grid::dimensions; ++i )
    cells[i] = grid.cells(i);
  double max = *( std::max_element( x.begin(), x.end() ) );
  double min = *( std::min_element( x.begin(), x.end() ) );
 
  CImg<double> goutput( cells[0], cells[1], cells[2], 1 );
  for(size_t i=0; i<grid_size; ++i)
    goutput[i] = (x[i] - min) / (max - min) * 255;
  goutput.save("pressure.tif");

  timer = clock();
  cout << "Time for output: " << (double)(timer-timer_begin)/CLOCKS_PER_SEC << endl;
  timer_begin = clock();

 return 0;
}
